﻿/*!
 *  @mainpage DHWR APIs Document
 *
 *  @section intro_sec Introduction
 *  본 문서는 SELVAS AI의 필기인식 솔루션인 DioPen의 인식엔진(DHWR)의 API 가이드이다. \n
 *  DHWR은 표준 C언어 기반의 라이브러리로 터치 스크린에 기반한 임베디드 플랫폼에 적용되고 있다.
 *
 *  @section install_sec Setting
 *  @subsection event_sec Touch Event
 *  필기 입력기 구현을 위해서는 다음과 같은 터치 이벤트 사양이 요구된다.
 *    - Touch Down : 펜을 터치 디바이스상에 처음 대는 순간 발생하는 이벤트.
 *    - Touch Move : 펜을 터치 디바이스에서 떼지 않은 상태로 있을 경우 발생하는 이벤트.
 *    - Touch Up : 펜을 터치 디바이스 상에서 떼는 순간 발생하는 이벤트. Touch Down이 발생한 경우라면 항상 Touch Up이 발생하여야 한다. 즉, 사용자가 필기 도중 터치 패드 영역을 벗어난 경우에도 드라이버에서는 Touch Up 이벤트를 발생 시켜야 한다.
 *
 *  @subsection samplingrate_sec Sampling Rate
 *  'Sampling Rate’란 터치 디바이스에서 터치 이벤트를 체크하는 시간(or 양)을 말한다. \n
 *  필기 입력기 적용을 위해서는 보통 10~40mSec(초당 25~100개의 터치이벤트) 정도로 조정한다. \n
 *  유의할 점은 Sampling Rate가 40mSec를 넘게 되면 필기한 모양이 자연스럽지 못하고 인식률도 떨어지게 된다. 대개의 경우 10~20mSec를 권장한다.
 *
 *  @subsection delaytime_sec Delay time
 *  인식 대기 시간은 필기 입력이 끝났다고 판단하는데 걸리는 시간이다. \n
 *  즉, 터치 디바이스에서 펜을 뗀 후 일정 시간 동안 필기 입력이 없으면 사용자의 입력이 끝났다고 판단하고 인식 엔진을 호출하게 된다. \n
 *  인식 대기 시간은 설정 메뉴를 두어 사용자로 하여 적당한 값을 선택할 수 있도록 할 수도 있으며, 일반적으로 300~500mSec의 값을 사용한다. \n
 *  인식 대상 문자를 지정하는 경우 대상 문자의 종류에 따라 차등 적용할 수도 있다.(예, 한글 - 400mSec, 영숫자 - 200mSec) \n
 *  물론, 인식 대기 시간을 활용하지 않고 특정 버튼을 두어 시간에 관계없이 버튼을 클릭하면 바로 인식 엔진을 호출하게끔 구현할 수도 있다.
 *
 *  @section porting_sec Porting
 *  포팅에 필요한 파일들은 다음과 같다.
 *    - dhwr.h - main API function 등이 선언되어 있다.
 *    - dhwr_define.h - 각종 definition
 *    - dhwr_gesture.h - gesture 관련 definition
 *    - dhwr_ink.h - ink 관련 definition
 *    - dhwr_result.h - result 관련 definition
 *    - dhwr_setting.h - setting 관련 definition
 *    - libdhwr.so, libdhwr-base.so, libdhwr-core.so : 필기인식 엔진 Main 라이브러리
 *    - llibdhwr-xxxx.so : 동적으로 로드 되는 Sub 라이브러리, 파일의 path를 DHWRSetExternalLibraryPath()의 파라미터로 설정해야 한다.
 *    - xx_XX.hdb : 각 언어의 인식을 위한 리소스 파일, 파일의 path를 DHWRSetExternalResourcePath()의 파라미터로 설정해야 한다.
 *    - dhwr.key : License를 위한 key 파일로써, key파일이 없으면 필기인식 엔진은 동작하지 않는다. 파일명을 포함한 path를 DHWRCreate()의 파라미터로 설정해야 한다.
 *  @section link_sec Link
 *  SELVAS AI http://www.selvasai.com/
 */

/*!
 *  @file  dhwr.h
 *  @date  2011/10/12
 *  @author  shchoi
 *
 *  Copyright (c) 2016 by SELVAS AI Inc. All rights reserved.
 */

/*! 
 *  @example example.c
 *  DHWR API들을 이용하여 필기인식기를 구현하는 예제
 */

#ifndef __DHWR_H__
#define __DHWR_H__

#include "dhwr_define.h"
#include "dhwr_gesture.h"
#include "dhwr_ink.h"
#include "dhwr_result.h"
#include "dhwr_setting.h"

#ifdef __cplusplus
extern "C" {
#endif

/*!
 *  @brief 엔진 내부 오류 코드 값
 */
typedef enum {
  //! 성공
  ERR_SUCCESS        = 0,
  //! 인식결과가 없음
  ERR_NORESULT,
  //! Null Pointer를 참조함
  ERR_NULL_POINTER,
  //! memory access 범위를 벗어남 fatal error
  ERR_OUTOFMEMORY,
  //! value or size 범위를 벗어남 exception
  ERR_OUTOFRANGE,
  //! 입력된 데이터가 없음
  ERR_EMPTY_INK,
  //! 입력된 인자가 잘못됨
  ERR_INVALID_ARGUMENTS,
  //! 설정된 인식모델이 비정상적임
  ERR_INVALID_MODEL,
  //! 잘못된 객체에 접근
  ERR_INVALID_INSTANCE,
  //! 데모 기간/횟수 만료
  ERR_EXPIRE_DEMO,
  //! 엔진이 실행 중
  ERR_ENGINE_BUSY,
  //! 인증 실패
  ERR_AUTHORIZATION_FAIL,
  //! 엔진이 이미 존재함
  ERR_ALREADY_EXIST
} DHWR_ERROR_CODE;

/* ----------------------------------------------------------------------
 * Functions
 * All functions return combination of DHWR_ERROR_CODE flags
 * -------------------------------------------------------------------- */

/**
 * @defgroup  Main Main APIs
 * @brief     엔진 사용에 필수적인 함수
 */

/*@{*/

/*!
 *  @brief 인식엔진의 인스턴스를 생성함
 *  @param [in] key_path 파일명을 포함한 License key 파일의 경로
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 *  @code
 *  unsigned long result = ERR_SUCCESS;
 *  result = DHWRCreate("/usr/local/dhwr.key");
 *  @endcode
 */
DHWR_API unsigned long DHWRCreate(const char* key_path);

/*!
 *  @brief 인식모드 속성 값을 설정함
 *  @param [in] setting 설정이 완료된 Setting Object
 *  @see DHWRSettingObject
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned int DHWRSetAttribute(DHWRSettingObject setting);

/*!
 *  @brief 인식모드 속성 값에 따라 입력 데이터를 인식함
 *  @param [in] ink 좌표가 입력된 Ink Object
 *  @param [out] context 인식된 결과가 저장된 Result Object
 *  @see DHWRInkObject, DHWRResultObject
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned long DHWRecognize(DHWRInkObject ink, DHWRResultObject context);

/*!
 *  @brief 인식엔진의 인스턴스를 소멸시킴
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned long DHWRClose(void);

/*@}*/

/**
 * @defgroup Option Optional APIs
 */

/*@{*/

/*!
 *  @brief  엔진내부에서 사용하는 외부리소스(ko_KR.hdb, en_US.hdb, ...)의 경로 설정
 *  @param [in] path 리소스 경로
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 *  @code
 *  unsigned long result = ERR_SUCCESS;
 *  result = DHWRSetExternalResourcePath("/usr/local/hdb");
 *  @endcode
 */
DHWR_API unsigned long DHWRSetExternalResourcePath(const char* path);

/*!
 *  @brief  엔진내부에서 사용하는 외부라이브러리(libdhwr-hwfs.so, ...)의 경로 설정
 *  @param [in] path 외부라이브러리의 경로
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 *  @code
 *  unsigned long result = ERR_SUCCESS;
 *  result = DHWRSetExternalLibraryPath("/usr/local/lib");
 *  @endcode
 */
DHWR_API unsigned long DHWRSetExternalLibraryPath(const char* path);

/*!
 *  @brief 특정 타입의 파라미터 값을 가져옴
 *  @param [in] type 파라미터 타입
 *  @param [out] param 특정 파리미터 타입의 포인터
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned long DHWRGetParam(const unsigned long type, void * param);

/*!
 *  @brief 특정 타입의 파라미터 값을 설정함
 *  @param [in] type 파라미터 타입
 *  @param [in] param 특정 파리미터 타입의 포인터
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned long DHWRSetParam(const unsigned long type, const void * param);

/*!
 *  @brief 엔진의 빌드넘버를 가져옴
 *  @param [out] revision 엔진 빌드넘버
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned long DHWRGetRevision(char ** revision);

/*!
 *  @brief License key에 설정된 엔진의 due date를 가져온다
 *  @remarks License key의 due date가 존재하지 않을 경우 due date는 초기 값 0이다
 *  @param [out] due_date License key의 due date, 년월일을 나타내는 8자리의 integer 값 (ex 20161206)
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned long DHWRGetDueDate(int* due_date);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif  // __DHWR_H__
